# BSD 3-Clause License

# Copyright (c) 2021, Muhammad Asad (masadcv@gmail.com)
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.

# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from typing import List
import torch
import FastGeodisCpp


def generalised_geodesic2d(
    image: torch.Tensor, 
    softmask: torch.Tensor, 
    v: float, 
    lamb: float, 
    iter: int = 2
):
    r"""Computes Generalised Geodesic Distance using FastGeodis raster scanning.
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU or GPU depending on Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.generalised_geodesic2d(
        image, softmask, v, lamb, 1 - lamb, iter
    )


def generalised_geodesic3d(
    image: torch.Tensor,
    softmask: torch.Tensor,
    spacing: List,
    v: float,
    lamb: float,
    iter: int = 4,
):
    r"""Computes Generalised Geodesic Distance using FastGeodis raster scanning.
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU or GPU depending on Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.generalised_geodesic3d(
        image, softmask, spacing, v, lamb, 1 - lamb, iter
    )


def signed_generalised_geodesic2d(
    image: torch.Tensor, 
    softmask: torch.Tensor, 
    v: float, 
    lamb: float, 
    iter: int = 2
):
    r"""Computes Signed Generalised Geodesic Distance using FastGeodis raster scanning.
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU or GPU depending on Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.signed_generalised_geodesic2d(
        image, softmask, v, lamb, 1 - lamb, iter
    )


def signed_generalised_geodesic3d(
    image: torch.Tensor,
    softmask: torch.Tensor,
    spacing: List,
    v: float,
    lamb: float,
    iter: int = 4,
):
    r"""Computes Signed Generalised Geodesic Distance using FastGeodis raster scanning.
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU or GPU depending on Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.signed_generalised_geodesic3d(
        image, softmask, spacing, v, lamb, 1 - lamb, iter
    )


def generalised_geodesic2d_toivanen(
    image: torch.Tensor, 
    softmask: torch.Tensor, 
    v: float, 
    lamb: float, 
    iter: int = 2
):
    r"""Computes Generalised Geodesic Distance using Toivanen's raster scanning method from:

    Toivanen, Pekka J. 
    "New geodosic distance transforms for gray-scale images." 
    Pattern Recognition Letters 17.5 (1996): 437-450.
    
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.generalised_geodesic2d_toivanen(
        image, softmask, v, lamb, 1 - lamb, iter
    )


def generalised_geodesic3d_toivanen(
    image: torch.Tensor,
    softmask: torch.Tensor,
    spacing: List,
    v: float,
    lamb: float,
    iter: int = 4,
):
    r"""Computes Generalised Geodesic Distance using Toivanen's raster scanning method from:

    Toivanen, Pekka J. 
    "New geodosic distance transforms for gray-scale images." 
    Pattern Recognition Letters 17.5 (1996): 437-450.
    
    For more details on generalised geodesic distance, check the following reference:


    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.generalised_geodesic3d_toivanen(
        image, softmask, spacing, v, lamb, 1 - lamb, iter
    )

def signed_generalised_geodesic2d_toivanen(
    image: torch.Tensor, 
    softmask: torch.Tensor, 
    v: float, 
    lamb: float, 
    iter: int = 2
):
    r"""Computes Signed Generalised Geodesic Distance using Toivanen's raster scanning method from:

    Toivanen, Pekka J. 
    "New geodosic distance transforms for gray-scale images." 
    Pattern Recognition Letters 17.5 (1996): 437-450.

    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.signed_generalised_geodesic2d_toivanen(
        image, softmask, v, lamb, 1 - lamb, iter
    )


def signed_generalised_geodesic3d_toivanen(
    image: torch.Tensor,
    softmask: torch.Tensor,
    spacing: List,
    v: float,
    lamb: float,
    iter: int = 4,
):
    r"""Computes Signed Generalised Geodesic Distance using Toivanen's raster scanning method from:

    Toivanen, Pekka J. 
    "New geodosic distance transforms for gray-scale images." 
    Pattern Recognition Letters 17.5 (1996): 437-450.

    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.signed_generalised_geodesic3d_toivanen(
        image, softmask, spacing, v, lamb, 1 - lamb, iter
    )

def generalised_geodesic2d_fastmarch(
    image: torch.Tensor, 
    softmask: torch.Tensor, 
    v: float, 
    lamb: float
):
    r"""Computes Generalised Geodesic Distance using Fast Marching method from:

    Sethian, James A. 
    "Fast marching methods." 
    SIAM review 41.2 (1999): 199-235.
    
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.generalised_geodesic2d_fastmarch(
        image, softmask, v, lamb, 1 - lamb
    )


def generalised_geodesic3d_fastmarch(
    image: torch.Tensor,
    softmask: torch.Tensor,
    spacing: List,
    v: float,
    lamb: float
):
    r"""Computes Generalised Geodesic Distance using Fast Marching method from:

    TSethian, James A. 
    "Fast marching methods." 
    SIAM review 41.2 (1999): 199-235.
    
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.generalised_geodesic3d_fastmarch(
        image, softmask, spacing, v, lamb, 1 - lamb
    )

def signed_generalised_geodesic2d_fastmarch(
    image: torch.Tensor, 
    softmask: torch.Tensor, 
    v: float, 
    lamb: float
):
    r"""Computes Signed Generalised Geodesic Distance using Fast Marching method from:

    Sethian, James A. 
    "Fast marching methods." 
    SIAM review 41.2 (1999): 199-235.
    
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.signed_generalised_geodesic2d_fastmarch(
        image, softmask, v, lamb, 1 - lamb
    )


def signed_generalised_geodesic3d_fastmarch(
    image: torch.Tensor,
    softmask: torch.Tensor,
    spacing: List,
    v: float,
    lamb: float
):
    r"""Computes Signed Generalised Geodesic Distance using Fast Marching method from:

    Sethian, James A. 
    "Fast marching methods." 
    SIAM review 41.2 (1999): 199-235.
    
    For more details on generalised geodesic distance, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.signed_generalised_geodesic3d_fastmarch(
        image, softmask, spacing, v, lamb, 1 - lamb
    )

def GSF2d(
    image: torch.Tensor,
    softmask: torch.Tensor,
    theta: float,
    v: float,
    lamb: float,
    iter: int,
):
    r"""Computes Geodesic Symmetric Filtering (GSF) using FastGeodis raster scanning.
    For more details on GSF, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU or GPU depending on Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.GSF2d(image, softmask, theta, v, lamb, iter)


def GSF3d(
    image: torch.Tensor,
    softmask: torch.Tensor,
    theta: float,
    spacing: List,
    v: float,
    lamb: float,
    iter: int,
):
    r"""Computes Geodesic Symmetric Filtering (GSF) using FastGeodis raster scanning.
    For more details on GSF, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU or GPU depending on Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.GSF3d(image, softmask, theta, spacing, v, lamb, iter)

def GSF2d_toivanen(
    image: torch.Tensor,
    softmask: torch.Tensor,
    theta: float,
    v: float,
    lamb: float,
    iter: int,
):
    r"""Computes Geodesic Symmetric Filtering (GSF) using Toivanen's raster scanning method from:

    Toivanen, Pekka J. 
    "New geodosic distance transforms for gray-scale images." 
    Pattern Recognition Letters 17.5 (1996): 437-450.

    For more details on GSF, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.GSF2d_toivanen(image, softmask, theta, v, lamb, iter)


def GSF3d_toivanen(
    image: torch.Tensor,
    softmask: torch.Tensor,
    theta: float,
    spacing: List,
    v: float,
    lamb: float,
    iter: int,
):
    r"""Computes Geodesic Symmetric Filtering (GSF) using Toivanen's raster scanning method from:

    Toivanen, Pekka J. 
    "New geodosic distance transforms for gray-scale images." 
    Pattern Recognition Letters 17.5 (1996): 437-450.
    
    For more details on GSF, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance
        iter: number of passes of the iterative distance transform method

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.GSF3d_toivanen(image, softmask, theta, spacing, v, lamb, iter)

def GSF2d_fastmarch(
    image: torch.Tensor,
    softmask: torch.Tensor,
    theta: float,
    v: float,
    lamb: float
):
    r"""Computes Geodesic Symmetric Filtering (GSF) using Fast Marching method from:

    Sethian, James A. 
    "Fast marching methods." 
    SIAM review 41.2 (1999): 199-235.

    For more details on GSF, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.GSF2d_fastmarch(image, softmask, theta, v, lamb)


def GSF3d_fastmarch(
    image: torch.Tensor,
    softmask: torch.Tensor,
    theta: float,
    spacing: List,
    v: float,
    lamb: float,
):
    r"""Computes Geodesic Symmetric Filtering (GSF) using Fast Marching method from:

    Sethian, James A. 
    "Fast marching methods." 
    SIAM review 41.2 (1999): 199-235.

    For more details on GSF, check the following reference:

    Criminisi, Antonio, Toby Sharp, and Andrew Blake.
    "Geos: Geodesic image segmentation."
    European Conference on Computer Vision, Berlin, Heidelberg, 2008.

    The function expects input as torch.Tensor, which can be run on CPU only using Tensor's device location

    Args:
        image: input image, can be grayscale or multiple channels.
        softmask: softmask in range [0, 1] with seed information.
        spacing: spacing for 3D data
        v: weighting factor for establishing relationship between unary and spatial distances.
        lamb: weighting factor between 0.0 and 1.0. 0.0 returns euclidean distance, whereas 1.0 returns geodesic distance

    Returns:
        torch.Tensor with distance transform
    """
    return FastGeodisCpp.GSF3d_fastmarch(image, softmask, theta, spacing, v, lamb)



from gettext import install
import glob
import os
import re
import sys
import warnings

import pkg_resources
from setuptools import find_packages, setup

FORCE_CUDA = os.getenv("FORCE_CUDA", "0") == "1"

BUILD_CPP = BUILD_CUDA = False
TORCH_VERSION = 0
try:
    import torch

    print(f"setup.py with torch {torch.__version__}")
    from torch.utils.cpp_extension import BuildExtension, CppExtension

    BUILD_CPP = True
    from torch.utils.cpp_extension import CUDA_HOME, CUDAExtension

    BUILD_CUDA = (CUDA_HOME is not None) if torch.cuda.is_available() else FORCE_CUDA

    _pt_version = pkg_resources.parse_version(torch.__version__)._version.release
    if _pt_version is None or len(_pt_version) < 3:
        raise AssertionError("unknown torch version")
    TORCH_VERSION = (
        int(_pt_version[0]) * 10000 + int(_pt_version[1]) * 100 + int(_pt_version[2])
    )
except (ImportError, TypeError, AssertionError, AttributeError) as e:
    warnings.warn(f"extension build skipped: {e}")
finally:
    print(
        f"BUILD_CPP={BUILD_CPP}, BUILD_CUDA={BUILD_CUDA}, TORCH_VERSION={TORCH_VERSION}."
    )


def torch_parallel_backend():
    try:
        match = re.search(
            "^ATen parallel backend: (?P<backend>.*)$",
            torch._C._parallel_info(),
            re.MULTILINE,
        )
        if match is None:
            return None
        backend = match.group("backend")
        if backend == "OpenMP":
            return "AT_PARALLEL_OPENMP"
        if backend == "native thread pool":
            return "AT_PARALLEL_NATIVE"
        if backend == "native thread pool and TBB":
            return "AT_PARALLEL_NATIVE_TBB"
    except (NameError, AttributeError):  # no torch or no binaries
        warnings.warn("Could not determine torch parallel_info.")
    return None


def omp_flags():
    if sys.platform == "win32":
        return ["/openmp"]
    if sys.platform == "darwin":
        # https://stackoverflow.com/questions/37362414/
        # return ["-fopenmp=libiomp5"]
        return []
    return ["-fopenmp"]


def get_extensions():
    # this_dir = os.path.dirname(os.path.abspath(__file__))
    # ext_dir = os.path.join(this_dir, "src")
    ext_dir = "FastGeodis"
    include_dirs = [ext_dir]

    source_cpu = glob.glob(os.path.join(ext_dir, "**", "*.cpp"), recursive=True)
    source_cuda = glob.glob(os.path.join(ext_dir, "**", "*.cu"), recursive=True)

    extension = None
    define_macros = [(f"{torch_parallel_backend()}", 1)]
    extra_compile_args = {}
    extra_link_args = []
    sources = source_cpu
    if BUILD_CPP:
        extension = CppExtension
        extra_compile_args.setdefault("cxx", [])
        if torch_parallel_backend() == "AT_PARALLEL_OPENMP":
            extra_compile_args["cxx"] += omp_flags()
        extra_link_args = omp_flags()
    if BUILD_CUDA:
        extension = CUDAExtension
        sources += source_cuda
        define_macros += [("WITH_CUDA", None)]
        extra_compile_args = {"cxx": [], "nvcc": []}
        if torch_parallel_backend() == "AT_PARALLEL_OPENMP":
            extra_compile_args["cxx"] += omp_flags()
    if extension is None or not sources:
        return []  # compile nothing
    
    # compile release
    extra_compile_args["cxx"] += ["-g0"]
    
    ext_modules = [
        extension(
            name="FastGeodisCpp",
            sources=sources,
            include_dirs=include_dirs,
            define_macros=define_macros,
            extra_compile_args=extra_compile_args,
            extra_link_args=extra_link_args,
        )
    ]
    return ext_modules


with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fp:
    install_requires = fp.read().splitlines()

setup(
    name="FastGeodis",
    version="1.0.0rc9",
    description="Fast Implementation of Generalised Geodesic Distance Transform for CPU (OpenMP) and GPU (CUDA)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/masadcv/FastGeodis",
    author="Muhammad Asad",
    author_email="muhammad.asad@kcl.ac.uk",
    license="BSD-3-Clause License",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
    ],
    install_requires=install_requires,
    cmdclass={
        "build_ext": BuildExtension
    },  # .with_options(no_python_abi_suffix=True)},
    packages=find_packages(exclude=("data", "docs", "examples", "scripts", "tests")),
    zip_safe=False,
    ext_modules=get_extensions(),
)
